@@ -32,6 +32,11 @@
 
 local NAME = "NSE";
 
+-- Script Scan phases.
+local NSE_PRE_SCAN  = "NSE_PRE_SCAN";
+local NSE_SCAN      = "NSE_SCAN";
+local NSE_POST_SCAN = "NSE_POST_SCAN";
+
 -- String keys into the registry (_R), for data shared with nse_main.cc.
 local YIELD = "NSE_YIELD";
 local BASE = "NSE_BASE";
@@ -39,7 +44,19 @@ local WAITING_TO_RUNNING = "NSE_WAITING_TO_RUNNING";
 local DESTRUCTOR = "NSE_DESTRUCTOR";
 local SELECTED_BY_NAME = "NSE_SELECTED_BY_NAME";
 
-local _R = debug.getregistry(); -- The registry
+-- This is a limit on the number of script instance threads running at once. It
+-- exists only to limit memory use when there are many open ports. It doesn't
+-- count worker threads started by scripts.
+local CONCURRENCY_LIMIT = 1000;
+
+-- Table of different supported rules.
+local NSE_SCRIPT_RULES = {
+  prerule = "prerule",
+  hostrule = "hostrule",
+  portrule = "portrule",
+  postrule = "postrule",
+};
+
 local _G = _G;
 
 local assert = assert;
@@ -60,15 +77,24 @@ local tostring = tostring;
 local type = type;
 local unpack = unpack;
 
+local coroutine = require "coroutine";
 local create = coroutine.create;
 local resume = coroutine.resume;
 local status = coroutine.status;
 local yield = coroutine.yield;
+local wrap = coroutine.wrap;
 
+local debug = require "debug";
 local traceback = debug.traceback;
+local _R = debug.getregistry();
 
+local io = require "io";
+local open = io.open;
+
+local math = require "math";
 local max = math.max;
 
+local string = require "string";
 local byte = string.byte;
 local find = string.find;
 local format = string.format;
@@ -77,6 +103,7 @@ local lower = string.lower;
 local match = string.match;
 local sub = string.sub;
 
+local table = require "table";
 local concat = table.concat;
 local insert = table.insert;
 local remove = table.remove;
@@ -89,9 +116,14 @@ local cnse, rules = ...; -- The NSE C library and Script Rules
 do -- Append the nselib directory to the Lua search path
   local t, path = assert(cnse.fetchfile_absolute("nselib/"));
   assert(t == "directory", "could not locate nselib directory!");
-  package.path = package.path..";"..path.."?.lua";
+  package.path = path.."?.lua;"..package.path;
 end
 
+local script_database_type, script_database_path =
+    cnse.fetchfile_absolute(cnse.script_dbpath);
+local script_database_update = cnse.scriptupdatedb;
+local script_help = cnse.scripthelp;
+
 local stdnse = require "stdnse";
 
 (require "strict")() -- strict global checking
@@ -139,6 +171,7 @@ end
 
 local log_write, verbosity, debugging =
     nmap.log_write, nmap.verbosity, nmap.debugging;
+local log_write_raw = cnse.log_write;
 
 local function print_verbose (level, fmt, ...)
   if verbosity() >= assert(tonumber(level)) or debugging() > 0 then
@@ -181,14 +214,77 @@ do
   -- Outputs debug information at level 1 or higher.
   -- Changes "%THREAD" with an appropriate identifier for the debug level
   function Thread:d (fmt, ...)
-    if debugging() > 1 then
-      print_debug(1, gsub(fmt, "%%THREAD", self.info), ...);
+    local against;
+    if self.host and self.port then
+      against = " against "..self.host.ip..":"..self.port.number;
+    elseif self.host then
+      against = " against "..self.host.ip;
     else
-      print_debug(1, gsub(fmt, "%%THREAD", self.short_basename), ...);
+      against = "";
+    end
+    if debugging() > 1 then
+      fmt = gsub(fmt, "%%THREAD_AGAINST", self.info..against);
+      fmt = gsub(fmt, "%%THREAD", self.info);
+    else
+      fmt = gsub(fmt, "%%THREAD_AGAINST", self.short_basename..against);
+      fmt = gsub(fmt, "%%THREAD", self.short_basename);
+    end
+    print_debug(1, fmt, ...);
+  end
+
+  -- Sets scripts output. Variable result is a string.
+  function Thread:set_output(result)
+    if self.type == "prerule" or self.type == "postrule" then
+      cnse.script_set_output(self.id, result);
+    elseif self.type == "hostrule" then
+      cnse.host_set_output(self.host, self.id, result);
+    elseif self.type == "portrule" then
+      cnse.port_set_output(self.host, self.port, self.id, result);
     end
   end
 
-  function Thread:close ()
+  -- prerule/postrule scripts may be timed out in the future
+  -- based on start time and script lifetime?
+  function Thread:timed_out ()
+    if self.type == "hostrule" or self.type == "portrule" then
+      return cnse.timedOut(self.host);
+    end
+    return nil;
+  end
+
+  function Thread:start_time_out_clock ()
+    if self.type == "hostrule" or self.type == "portrule" then
+      cnse.startTimeOutClock(self.host);
+    end
+  end
+
+  function Thread:stop_time_out_clock ()
+    if self.type == "hostrule" or self.type == "portrule" then
+      cnse.stopTimeOutClock(self.host);
+    end
+  end
+
+  -- Register scripts in the timeouts list to track their timeouts.
+  function Thread:start (timeouts)
+    self:d("Starting %THREAD_AGAINST.");
+    if self.host then
+      timeouts[self.host] = timeouts[self.host] or {};
+      timeouts[self.host][self.co] = true;
+    end
+  end
+
+  -- Remove scripts from the timeouts list and call their
+  -- destructor handles.
+  function Thread:close (timeouts, result)
+    self.error = result;
+    if self.host then
+      timeouts[self.host][self.co] = nil;
+      -- Any more threads running for this script/host?
+      if not next(timeouts[self.host]) then
+        self:stop_time_out_clock();
+        timeouts[self.host] = nil;
+      end
+    end
     local ch = self.close_handlers;
     for key, destructor_t in pairs(ch) do
       destructor_t.destructor(destructor_t.thread, key);
@@ -204,12 +300,15 @@ do
   -- Returns:
   --   thread  The thread (class) is returned, or nil.
   function Script:new_thread (rule, ...)
-    assert(rule == "hostrule" or rule == "portrule");
+    local script_type = assert(NSE_SCRIPT_RULES[rule]);
     if not self[rule] then return nil end -- No rule for this script?
     local file_closure = self.file_closure;
-    local env = setmetatable({
-        filename = self.filename,
-      }, {__index = _G});
+    local env = {
+        SCRIPT_PATH = self.filename,
+        SCRIPT_NAME = self.short_basename,
+        SCRIPT_TYPE = script_type,
+    };
+    setmetatable(env, {__index = _G});
     setfenv(file_closure, env);
     local unique_value = {}; -- to test valid yield
     local function main (...)
@@ -221,19 +320,21 @@ do
     -- same Lua thread the action and rule functions will execute in.
     local co = create(main);
     local s, value, rule_return = resume(co, ...);
+    setfenv(file_closure, _G); -- reset the environment
     if s and value ~= unique_value then
       print_debug(1,
     "A thread for %s yielded unexpectedly in the file or %s function:\n%s\n",
           self.filename, rule, traceback(co));
     elseif s and rule_return then
-      local thread = setmetatable({
+      local thread = {
         co = co,
         env = env,
         identifier = tostring(co),
         info = format("'%s' (%s)", self.short_basename, tostring(co));
-        type = rule == "hostrule" and "host" or "port",
+        type = script_type,
         close_handlers = {},
-      }, {
+      };
+      setmetatable(thread, {
         __metatable = Thread,
         __index = function (thread, k) return Thread[k] or self[k] end
       }); -- Access to the parent Script
@@ -267,10 +368,11 @@ do
     end
     local file_closure = assert(loadfile(filename));
     -- Give the closure its own environment, with global access
-    local env = setmetatable({
+    local env = {
       filename = filename,
       dependencies = {},
-    }, {__index = _G});
+    };
+    setmetatable(env, {__index = _G});
     setfenv(file_closure, env);
     local co = create(file_closure); -- Create a garbage thread
     assert(resume(co)); -- Get the globals it loads in env
@@ -284,10 +386,19 @@ do
             type(field).."', expected type '"..t.."'");
       end
     end
-    -- Check one of two required rule functions exists
-    local hostrule, portrule = rawget(env, "hostrule"), rawget(env, "portrule");
-    assert(type(hostrule) == "function" or type(portrule) == "function",
-        filename.." is missing a required function: 'hostrule' or 'portrule'");
+    -- Check the required rule functions
+    local rules = {}
+    for rule in pairs(NSE_SCRIPT_RULES) do
+      local rulef = rawget(env, rule);
+      assert(type(rulef) == "function" or rulef == nil,
+          rule.." must be a function!");
+      rules[rule] = rulef;
+    end
+    assert(next(rules), filename.." is missing required function: 'rule'");
+    local prerule = rules.prerule;
+    local hostrule = rules.hostrule;
+    local portrule = rules.portrule;
+    local postrule = rules.postrule;
     -- Assert that categories is an array of strings
     for i, category in ipairs(rawget(env, "categories")) do
       assert(type(category) == "string", 
@@ -299,7 +410,7 @@ do
         filename.." has non-string entries in the 'dependencies' array");
     end
     -- Return the script
-    return setmetatable({
+    local script = {
       filename = filename,
       basename = match(filename, "[/\\]([^/\\]-)$") or filename,
       short_basename = match(filename, "[/\\]([^/\\]-)%.nse$") or
@@ -307,16 +418,20 @@ do
                        filename,
       id = match(filename, "^.-[/\\]([^\\/]-)%.nse$") or filename,
       file_closure = file_closure,
-      hostrule = type(hostrule) == "function" and hostrule or nil,
-      portrule = type(portrule) == "function" and portrule or nil,
+      prerule = prerule,
+      hostrule = hostrule,
+      portrule = portrule,
+      postrule = postrule,
       args = {n = 0};
+      description = rawget(env, "description"),
       categories = rawget(env, "categories"),
       author = rawget(env, "author"),
       license = rawget(env, "license"),
       dependencies = rawget(env, "dependencies"),
       threads = {},
       selected_by_name = false,
-    }, {__index = Script, __metatable = Script});
+    };
+    return setmetatable(script, {__index = Script, __metatable = Script});
   end
 end
 
@@ -340,14 +455,7 @@ end
 local function get_chosen_scripts (rules)
   check_rules(rules);
 
-  local script_dbpath = cnse.script_dbpath;
-  local t, path = cnse.fetchfile_absolute(script_dbpath);
-  if not t then
-    print_verbose(1, "Creating non-existent script database.");
-    assert(cnse.updatedb(), "could not update script database!");
-    t, path = assert(cnse.fetchfile_absolute(script_dbpath));
-  end
-  local db_closure = assert(loadfile(path),
+  local db_closure = assert(loadfile(script_database_path),
     "database appears to be corrupt or out of date;\n"..
     "\tplease update using: nmap --script-updatedb");
 
@@ -469,8 +577,9 @@ local function get_chosen_scripts (rules)
         print_debug(2, "Script %s was selected by name.", script.filename);
         files_loaded[path] = true;
       elseif t == "directory" then
-        for i, file in ipairs(cnse.dump_dir(path)) do
-          if not files_loaded[file] then
+        for f in cnse.dir(path) do
+          local file = path .."/".. f
+          if find(f, "%.nse$") and not files_loaded[file] then
             chosen_scripts[#chosen_scripts+1] = Script.new(file);
             files_loaded[file] = true;
           end
@@ -517,27 +626,18 @@ end
 -- The main loop function for NSE. It handles running all the script threads.
 -- Arguments:
 --   threads  An array of threads (a runlevel) to run.
-local function run (threads)
+--   scantype  A string that indicates the current script scan phase.
+local function run (threads_iter, scantype)
   -- running scripts may be resumed at any time. waiting scripts are
   -- yielded until Nsock wakes them. After being awakened with
   -- nse_restore, waiting threads become pending and later are moved all
   -- at once back to running.
   local running, waiting, pending = {}, {}, {};
   local all = setmetatable({}, {__mode = "kv"}); -- base coroutine to Thread
-  -- hosts maps a host to a list of threads for that host.
-  local hosts, total = {}, 0;
-  local current;
-  local progress = cnse.scan_progress_meter(NAME);
-
-  print_debug(1, "NSE Script Threads (%d) running:", #threads);
-  while #threads > 0 do
-    local thread = remove(threads);
-    thread:d("Starting %THREAD against %s%s.", thread.host.ip,
-        thread.port and ":"..thread.port.number or "");
-    all[thread.co], running[thread.co], total = thread, thread, total+1;
-    hosts[thread.host] = hosts[thread.host] or {};
-    hosts[thread.host][thread.co] = true;
-  end
+  local current; -- The currently running Thread.
+  local total = 0; -- Number of threads, for record keeping.
+  local timeouts = {}; -- A list to save and to track scripts timeout.
+  local num_threads = 0; -- Number of script instances currently running.
 
   -- Map of yielded threads to the base Thread
   local yielded_base = setmetatable({}, {__mode = "kv"});
@@ -585,11 +685,14 @@ local function run (threads)
         current.parent.info, tostring(co));
     local thread = {
       co = co,
+      id = current.id,
       args = {n = select("#", ...), ...},
       host = current.host,
       port = current.port,
+      type = current.type,
       parent = current.parent,
       info = format("'%s' worker (%s)", current.short_basename, tostring(co));
+      close_handlers = {},
       -- d = function(...) end, -- output no debug information
     };
     local thread_mt = {
@@ -607,8 +710,36 @@ local function run (threads)
     return current.co;
   end);
 
+  while threads_iter and num_threads < CONCURRENCY_LIMIT do
+    local thread = threads_iter()
+    if not thread then
+      threads_iter = nil;
+      break;
+    end
+    all[thread.co], running[thread.co], total = thread, thread, total+1;
+    num_threads = num_threads + 1;
+    thread:start(timeouts);
+  end
+  if num_threads == 0 then
+    return
+  end
+
+  local progress = cnse.scan_progress_meter(NAME);
+
   -- Loop while any thread is running or waiting.
-  while next(running) or next(waiting) do
+  while next(running) or next(waiting) or threads_iter do
+    -- Start as many new threads as possible.
+    while threads_iter and num_threads < CONCURRENCY_LIMIT do
+      local thread = threads_iter()
+      if not thread then
+        threads_iter = nil;
+        break;
+      end
+      all[thread.co], running[thread.co], total = thread, thread, total+1;
+      num_threads = num_threads + 1;
+      thread:start(timeouts);
+    end
+
     local nr, nw = table_size(running), table_size(waiting);
     if cnse.key_was_pressed() then
       print_verbose(1, "Active NSE Script Threads: %d (%d waiting)\n",
@@ -632,37 +763,38 @@ local function run (threads)
       end
     end
 
-    -- Checked for timed-out hosts.
+    -- Checked for timed-out scripts and hosts.
     for co, thread in pairs(waiting) do
-      if cnse.timedOut(thread.host) then
-        waiting[co], all[co] = nil, nil;
-        thread:d("%THREAD %s%s timed out", thread.host.ip,
-            thread.port and ":"..thread.port.number or "");
-        thread:close();
+      if thread:timed_out() then
+        waiting[co], all[co], num_threads = nil, nil, num_threads-1;
+        thread:d("%THREAD %stimed out", thread.host
+            and format("%s%s ", thread.host.ip,
+                    thread.port and ":"..thread.port.number or "")
+            or "");
+        thread:close(timeouts, "timed out");
       end
     end
 
     for co, thread in pairs(running) do
       current, running[co] = thread, nil;
-      cnse.startTimeOutClock(thread.host);
+      thread:start_time_out_clock();
 
       local s, result = resume(co, unpack(thread.args, 1, thread.args.n));
       if not s then -- script error...
-        hosts[thread.host][co], all[co] = nil, nil;
-        thread:d("%THREAD against %s%s threw an error!\n%s\n",
-            thread.host.ip, thread.port and ":"..thread.port.number or "",
+        all[co], num_threads = nil, num_threads-1;
+        thread:d("%THREAD_AGAINST threw an error!\n%s\n",
             traceback(co, tostring(result)));
-        thread.error = result;
-        thread:close();
+        thread:close(timeouts, result);
       elseif status(co) == "suspended" then
         if result == NSE_YIELD_VALUE then
           waiting[co] = thread;
         else
-          thread:d("%THREAD yielded unexpectedly and cannot be rerun.");
+          all[co], num_threads = nil, num_threads-1;
+          thread:d("%THREAD yielded unexpectedly and cannot be resumed.");
           thread:close();
         end
       elseif status(co) == "dead" then
-        hosts[thread.host][co], all[co] = nil, nil;
+        all[co], num_threads = nil, num_threads-1;
         if type(result) == "string" then
           -- Escape any character outside the range 32-126 except for tab,
           -- carriage return, and line feed. This makes the string safe for
@@ -670,21 +802,12 @@ local function run (threads)
           result = gsub(result, "[^\t\r\n\032-\126]", function(a)
             return format("\\x%02X", byte(a));
           end);
-          if thread.type == "host" then
-            cnse.host_set_output(thread.host, thread.id, result);
-          else
-            cnse.port_set_output(thread.host, thread.port, thread.id, result);
-          end
+          thread:set_output(result);
         end
-        thread:d("Finished %THREAD against %s%s.", thread.host.ip,
-            thread.port and ":"..thread.port.number or "");
-        thread:close();
-      end
-
-      -- Any more threads running for this host?
-      if not next(hosts[thread.host]) then
-        cnse.stopTimeOutClock(thread.host);
+        thread:d("Finished %THREAD_AGAINST.");
+        thread:close(timeouts);
       end
+      current = nil;
     end
 
     cnse.nsock_loop(50); -- Allow nsock to perform any pending callbacks
@@ -699,6 +822,74 @@ local function run (threads)
   progress "endTask";
 end
 
+-- Format NSEDoc markup (e.g., including bullet lists and <code> sections) into
+-- a display string at the given indentation level. Currently this only indents
+-- the string and doesn't interpret any other markup.
+local function format_nsedoc(nsedoc, indent)
+  indent = indent or ""
+
+  return string.gsub(nsedoc, "([^\n]+)", indent .. "%1")
+end
+
+-- Return the NSEDoc URL for the script with the given id.
+local function nsedoc_url(id)
+  return string.format("%s/nsedoc/scripts/%s.html", cnse.NMAP_URL, id)
+end
+
+local function script_help_normal(chosen_scripts)
+  for i, script in ipairs(chosen_scripts) do
+    log_write_raw("stdout", "\n");
+    log_write_raw("stdout", string.format("%s\n", script.id));
+    log_write_raw("stdout", string.format("Categories: %s\n", table.concat(script.categories, " ")));
+    log_write_raw("stdout", string.format("%s\n", nsedoc_url(script.id)));
+    log_write_raw("stdout", format_nsedoc(script.description, "  "));
+  end
+end
+
+local function script_help_xml(chosen_scripts)
+  cnse.xml_start_tag("nse-scripts");
+  cnse.xml_newline();
+
+  local t, scripts_dir, nselib_dir
+  t, scripts_dir = cnse.fetchfile_absolute("scripts/")
+  assert(t == 'directory', 'could not locate scripts directory');
+  t, nselib_dir = cnse.fetchfile_absolute("nselib/")
+  assert(t == 'directory', 'could not locate nselib directory');
+  cnse.xml_start_tag("directory", { name = "scripts", path = scripts_dir });
+  cnse.xml_end_tag();
+  cnse.xml_newline();
+  cnse.xml_start_tag("directory", { name = "nselib", path = nselib_dir });
+  cnse.xml_end_tag();
+  cnse.xml_newline();
+
+  for i, script in ipairs(chosen_scripts) do
+    cnse.xml_start_tag("script", { filename = script.filename });
+    cnse.xml_newline();
+
+    cnse.xml_start_tag("categories");
+    for _, category in ipairs(script.categories) do
+      cnse.xml_start_tag("category");
+      cnse.xml_write_escaped(category);
+      cnse.xml_end_tag();
+    end
+    cnse.xml_end_tag();
+    cnse.xml_newline();
+
+    cnse.xml_start_tag("description");
+    cnse.xml_write_escaped(script.description);
+    cnse.xml_end_tag();
+    cnse.xml_newline();
+
+    -- script
+    cnse.xml_end_tag();
+    cnse.xml_newline();
+  end
+
+  -- nse-scripts
+  cnse.xml_end_tag();
+  cnse.xml_newline();
+end
+
 do -- Load script arguments (--script-args)
   local args = cnse.scriptargs or "";
 
@@ -762,64 +953,159 @@ do -- Load script arguments (--script-args)
   nmap.registry.args = parse_table("{"..args.."}", 1);
 end
 
+-- Update Missing Script Database?
+if script_database_type ~= "file" then
+  print_verbose(1, "Script Database missing, will create new one.");
+  script_database_update = true; -- force update
+end
+
+if script_database_update then
+  log_write("stdout", "Updating rule database.");
+  local t, path = cnse.fetchfile_absolute('scripts/'); -- fetch script directory
+  assert(t == 'directory', 'could not locate scripts directory');
+  script_database_path = path.."script.db";
+  local db = assert(open(script_database_path, 'w'));
+  local scripts = {};
+  for f in cnse.dir(path) do
+    if match(f, '%.nse$') then
+      scripts[#scripts+1] = path.."/"..f;
+    end
+  end
+  sort(scripts);
+  for i, script in ipairs(scripts) do
+    script = Script.new(script);
+    sort(script.categories);
+    db:write('Entry { filename = "', script.basename, '", ');
+    db:write('categories = {');
+    for j, category in ipairs(script.categories) do
+      db:write(' "', lower(category), '",');
+    end
+    db:write(' } }\n');
+  end
+  db:close();
+  log_write("stdout", "Script Database updated successfully.");
+end
+
 -- Load all user chosen scripts
 local chosen_scripts = get_chosen_scripts(rules);
 print_verbose(1, "Loaded %d scripts for scanning.", #chosen_scripts);
 for i, script in ipairs(chosen_scripts) do
-  print_debug(2, "Loaded '%s'.", script.basename);
+  print_debug(2, "Loaded '%s'.", script.filename);
+end
+
+if script_help then
+  script_help_normal(chosen_scripts);
+  script_help_xml(chosen_scripts);
 end
 
 -- main(hosts)
--- This is the main function we return to NSE (on the C side) which actually
--- runs a scan against an array of hosts. nse_main.cc gets this function
--- by loading and executing nse_main.lua.
+-- This is the main function we return to NSE (on the C side), nse_main.cc
+-- gets this function by loading and executing nse_main.lua. This
+-- function runs a script scan phase according to its arguments.
 -- Arguments:
 --   hosts  An array of hosts to scan.
-return function (hosts)
-  if #hosts > 1 then
-    print_verbose(1, "Script scanning %d hosts.", #hosts);
-  elseif #hosts == 1 then
-    print_verbose(1, "Script scanning %s.", hosts[1].ip);
+--   scantype A string that indicates the current script scan phase.
+--    Possible string values are:
+--      "SCRIPT_PRE_SCAN"
+--      "SCRIPT_SCAN"
+--      "SCRIPT_POST_SCAN"
+local function main (hosts, scantype)
+  -- Used to set up the runlevels.
+  local threads, runlevels = {}, {};
+
+  -- Every script thread has a table that is used in the run function
+  -- (the main loop of NSE).
+  -- This is the list of the thread table key/value pairs:
+  --  Key     Value
+  --  type    A string that indicates the rule type of the script.
+  --  co      A thread object to identify the coroutine.
+  --  parent  A table that contains the parent thread table (it self).
+  --  close_handlers
+  --          A table that contains the thread destructor handlers.
+  --  info    A string that contains the script name and the thread 
+  --            debug information.
+  --  args    A table that contains the arguments passed to scripts, 
+  --            arguments can be host and port tables.
+  --  env     A table that contains the global script environment:
+  --            categories, description, author, license, nmap table,
+  --            action function, rule functions, SCRIPT_PATH, 
+  --            SCRIPT_NAME, SCRIPT_TYPE (pre|host|port|post rule).
+  --  identifier
+  --          A string to identify the thread address.
+  --  host    A table that contains the target host information. This
+  --          will be nil for Pre-scanning and Post-scanning scripts.
+  --  port    A table that contains the target port information. This
+  --          will be nil for Pre-scanning and Post-scanning scripts.
+
+  local runlevels = {};
+  for i, script in ipairs(chosen_scripts) do
+    runlevels[script.runlevel] = runlevels[script.runlevel] or {};
+    insert(runlevels[script.runlevel], script);
   end
 
-  -- Set up the runlevels.
-  local threads, runlevels = {}, {};
-  for j, host in ipairs(hosts) do
-    -- Check hostrules for this host.
-    for i, script in ipairs(chosen_scripts) do
-      local thread = script:new_thread("hostrule", tcopy(host));
-      if thread then
-        local runlevel = thread.runlevel;
-        if threads[runlevel] == nil then insert(runlevels, runlevel); end
-        threads[runlevel] = threads[runlevel] or {};
-        insert(threads[runlevel], thread);
-        thread.args, thread.host = {n = 1, tcopy(host)}, host;
-      end
+  if (scantype == NSE_PRE_SCAN) then
+    print_verbose(1, "Script Pre-scanning.");
+  elseif (scantype == NSE_SCAN) then
+    if #hosts > 1 then
+      print_verbose(1, "Script scanning %d hosts.", #hosts);
+    elseif #hosts == 1 then
+      print_verbose(1, "Script scanning %s.", hosts[1].ip);
     end
-    -- Check portrules for this host.
-    for port in cnse.ports(host) do
-      for i, script in ipairs(chosen_scripts) do
-        local thread = script:new_thread("portrule", tcopy(host),
-            tcopy(port));
-        if thread then
-          local runlevel = thread.runlevel;
-          if threads[runlevel] == nil then insert(runlevels, runlevel); end
-          threads[runlevel] = threads[runlevel] or {};
-          insert(threads[runlevel], thread);
-          thread.args, thread.host, thread.port =
-              {n = 2, tcopy(host), tcopy(port)}, host, port;
+  elseif (scantype == NSE_POST_SCAN) then
+    print_verbose(1, "Script Post-scanning.");
+  end
+
+  for runlevel, scripts in ipairs(runlevels) do
+    -- This iterator is passed to the run function. It returns one new script
+    -- thread on demand until exhausted.
+    local function threads_iter ()
+      -- activate prerule scripts
+      if (scantype == NSE_PRE_SCAN) then
+        for _, script in ipairs(scripts) do
+           local thread = script:new_thread("prerule");
+           if thread then
+             thread.args = {n = 0};
+             yield(thread);
+           end
+        end
+      -- activate hostrule and portrule scripts
+      elseif (scantype == NSE_SCAN) then
+        -- Check hostrules for this host.
+        for j, host in ipairs(hosts) do
+          for _, script in ipairs(scripts) do
+            local thread = script:new_thread("hostrule", tcopy(host));
+            if thread then
+              thread.args, thread.host = {n = 1, tcopy(host)}, host;
+              yield(thread);
+            end
+          end
+          -- Check portrules for this host.
+          for port in cnse.ports(host) do
+            for _, script in ipairs(scripts) do
+              local thread = script:new_thread("portrule", tcopy(host), tcopy(port));
+              if thread then
+                thread.args, thread.host, thread.port = {n = 2, tcopy(host), tcopy(port)}, host, port;
+                yield(thread);
+              end
+            end
+          end
+        end
+        -- activate postrule scripts
+      elseif (scantype == NSE_POST_SCAN) then
+        for _, script in ipairs(scripts) do
+          local thread = script:new_thread("postrule");
+          if thread then
+            thread.args = {n = 0};
+            yield(thread);
+          end
         end
       end
     end
-  end
-
-  sort(runlevels);
-  for i, runlevel in ipairs(runlevels) do
-    print_verbose(1, "Starting runlevel %u (of %u) scan.", runlevel,
-        #runlevels);
-    run(threads[runlevel]);
+    print_verbose(1, "Starting runlevel %u (of %u) scan.", runlevel, #runlevels);
+    run(wrap(threads_iter), scantype)
   end
 
   collectgarbage "collect";
-  print_verbose(1, "Script Scanning completed.");
 end
+
+return main;
